#ifndef PANEL_NWCHEM_H
#define PANEL_NWCHEM_H

#ifndef PANEL_H
#include "Panel.h"
#endif

#ifndef DYNAMIC_SINGLETON_H
#include "Dynamic_singleton.h"
#endif

#ifndef UI_PANEL_NWCHEM_H
#include "ui_Panel_NWCHEM.h"
#endif

#include <QProcess>
#include <QFile>

namespace MM
{
class Model;
class Main_window;

class Panel_NWChem 
:
    public Panel,
    public Ui_Panel_NWChem,
    public Dynamic_singleton <Panel_NWChem>
{
    Q_OBJECT

    Model *         model_;
    QProcess *      process_;
    Text            NWChem_dir_;
    QFile           output_;
    Main_window *   player_;

    bool            read_atom_coords_flag_;
    bool            read_charges_flag_;
    bool            read_moments_flag_;
    bool            user_stop_flag_;
    bool            success_;
    int             optimization_it_;

public:    
    Panel_NWChem ();
   ~Panel_NWChem ();

public slots:
    void            show_options (const QString & text);

    void            run   ();
    void            tune  ();
    void            start ();
    void            stop  ();

    void            started ();
    void            finished (int exitCode, QProcess::ExitStatus exitStatus);

    void            read ();

protected slots:

protected:
    Text const &    NWChem_dir () const {return NWChem_dir_;}
    Text const &    setup_NWChem_dir ();

    void            read_atom_coords (char *buf);
    void            read_charges     (char *buf);
    void            read_moments     (char *buf);
    void            create_input     ();

    Text            basis ();
    Text            data  ();
    Text            operation ();
    Text            quality ();
    Text            spherical_or_cartesian () const;

    bool            not_semiempirical ();
    void            set_stop_state ();
    void            set_run_state  ();

    void            turn (bool on);
    void            closeEvent (QCloseEvent * event);
};

}//MM

#endif //PANEL_NWCHEM_H
